<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StreamWrapper\PublicStream;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Defines the image service interface.
 */
interface StandWithUkraineImageInterface extends TrustedCallbackInterface {

  /**
   * The cached data identifiers prefix.
   */
  const CACHE = 'standwithukraine:';

  /**
   * The flag colors.
   */
  const COLORS = [
    TRUE => [0x00, 0x5B, 0xBB],
    FALSE => [0xFF, 0xD5, 0x00],
  ];

  /**
   * The default ratio for an unknown theme.
   *
   * The first half of the space is for blue color and the second half space for
   * yellow color.
   */
  const UNDEFINED_RATIO = 50;

  /**
   * The category of supported media types.
   *
   * This text can not be added to the following constant directly because in
   * that case, Drupal Rector will define array items in that constant as
   * libraries.
   */
  const CATEGORY = 'image/';

  /**
   * The image filters.
   */
  const FILTERS = [
    FALSE => 'foreground',
    TRUE => 'background',
  ];

  /**
   * The supported media types.
   */
  const TYPES = [
    'png' => [
      self::CATEGORY . 'png',
    ],
    'ico' => [
      self::CATEGORY . 'vnd.microsoft.icon',
      self::CATEGORY . 'x-icon',
    ],
  ];

  /**
   * The supported location of files.
   */
  const STREAM = 'public://';

  /**
   * StandWithUkraineImage constructor.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory.
   * @param \Drupal\Core\StreamWrapper\PublicStream $stream
   *   The public stream wrapper.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The string translation.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme
   *   The theme manager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface|null $file_url_generator
   *   (optional) The file URL generator. Defaults to NULL.
   */
  public function __construct(
    CacheBackendInterface $cache,
    LoggerChannelFactoryInterface $logger_factory,
    PublicStream $stream,
    TranslationInterface $translation,
    ThemeManagerInterface $theme,
    FileUrlGeneratorInterface $file_url_generator = NULL
  );

  /**
   * Adds fields to form to configure ratio and filling type.
   *
   * @param int $ratio
   *   The default ratio.
   * @param bool $background
   *   TRUE, if the background should be styled instead of the foreground.
   *
   * @return array
   *   The renderable element.
   */
  public function form($ratio, $background);

  /**
   * Check if a file type is one from the list of processable.
   *
   * @param string $type
   *   The MIME type.
   *
   * @return bool
   *   TRUE if so.
   */
  public function isSupported($type);

  /**
   * Looking for images among renderable elements to modify them.
   *
   * @param array $elements
   *   The renderable elements tree.
   * @param array $variables
   *   The block theme variables.
   * @param int $ratio
   *   The percentage of flag first color.
   * @param bool $background
   *   TRUE, if the background should be styled instead of the foreground.
   * @param string|null $path
   *   (optional) The location of files. Defaults to NULL.
   * @param string|null $template
   *   (optional) The pattern of the main part of the file name. Defaults to NULL.
   * @param string|null $theme
   *   (optional) The theme name. Defaults to NULL.
   */
  public function search(
    array &$elements,
    array $variables,
    $ratio,
    $background,
    $path = NULL,
    $template = NULL,
    $theme = NULL
  );

}
